use Irssi;
use Irssi::Irc;
use strict;

use vars qw($VERSION %IRSSI);

$VERSION="0.3";
%IRSSI = (
	authors=> 'Mr. Test1, Mr. test2',
	contact=> 'foo@example.com',
	name=> 'test',
	description=> 'some description.',
	license=> 'GPL v2',
	url=> 'https://example.com/example',
);

